@@ -3,15 +3,13 @@ Used as base Mailchimp:
 https://github.com/mailchimp/mailchimp-marketing-python/blob/master/mailchimp_marketing/api_client.py
 """
 
-from typing import List, Dict
+from typing import List, Dict, Optional
 
 import datetime
 import requests
-import json
-import asyncio
 from tenacity import retry, wait_exponential, stop_after_attempt
 from shimoku_api_python.exceptions import ApiClientError
-from functools import wraps
+from pkg_resources import get_distribution
 
 import aiohttp
 import logging
@@ -24,13 +22,21 @@ class ApiClient(object):
     PRIMITIVE_TYPES = (float, int, bool, bytes, str)
 
     @logging_before_and_after(logging_level=logger.debug)
-    def __init__(self, universe_id: str, environment: str, config={}):
+    def __init__(self, environment: str, config=None):
+
+        self.cache_enabled = True
+
+        if config is None:
+            config = {}
+
         if environment == 'production':
             self.host = 'https://api.shimoku.io/external/v1/'
         elif environment == 'staging':
             self.host = 'https://api.staging.shimoku.io/external/v1/'
         elif environment == 'develop':
             self.host = 'https://api.develop.shimoku.io/external/v1/'
+        elif environment == 'guillermo':
+            self.host = 'https://wxauh7u2te.execute-api.eu-west-1.amazonaws.com/guillermo/external/v1/'
         else:
             raise ValueError(
                 f'The namespace must be either "production", "staging" or "develop | '
@@ -39,9 +45,7 @@ class ApiClient(object):
 
         # semaphor for async api calls
         self.semaphore_limit = 10
-        self.semaphore = asyncio.Semaphore(self.semaphore_limit)
-
-        self.host: str = f'{self.host}universe/{universe_id}/'
+        self.semaphore = None
 
         # DEFAULTS
         # Api key
@@ -59,6 +63,7 @@ class ApiClient(object):
             'User-Agent': 'Swagger-Codegen/0.0/python'
         }
         self.set_config(config)
+        self.call_counter = 0
 
         # Default vars
 
@@ -85,11 +90,12 @@ class ApiClient(object):
         self.timeout = config['timeout'] if 'timeout' in config.keys() else 120
 
     @logging_before_and_after(logging_level=logger.debug)
-    @retry(stop=stop_after_attempt(6), wait=wait_exponential(multiplier=2, min=1, max=16),
+    @retry(stop=stop_after_attempt(1), wait=wait_exponential(multiplier=2, min=1, max=16),
            before_sleep=my_before_sleep)
     async def call_api(
             self, resource_path, method, path_params=None, query_params=None,
-            header_params=None, body=None, collection_formats=None, **kwargs
+            header_params=None, body=None, collection_formats=None, limit: Optional[int] = None,
+            **kwargs
     ):
         """Create and call the API request with headers, params and others"""
         # header parameters
@@ -122,15 +128,12 @@ class ApiClient(object):
             url = url.replace('server', self.server)
 
         # perform request and return response
-        try:
-            async with self.semaphore:
-                return await self.request(
-                    method, url, query_params,
-                    headers=header_params, body=body,
-                )
-        except Exception as err:
-            logger.error(err)
-            raise ApiClientError(err)
+        async with self.semaphore:
+            return await self.request(
+                method, url, query_params,
+                headers=header_params, body=body,
+                limit=limit
+            )
 
     @logging_before_and_after(logging_level=logger.debug)
     def set_http_info(self, **kwargs):  # noqa: E501
@@ -179,12 +182,13 @@ class ApiClient(object):
 
     @logging_before_and_after(logging_level=logger.debug)
     async def query_element(
-            self, method: str, endpoint: str, **kwargs
+            self, method: str, endpoint: str, limit: Optional[int] = None, **kwargs
     ) -> Dict:
         """Retrieve an element if the endpoint exists
 
         :param method: examples are 'GET', 'POST', etc
         :param endpoint: example: 'business/{businessId}/app/{appId}
+        :param limit: limit the number of results returned
         """
         (
             query_params, header_params,
@@ -202,6 +206,7 @@ class ApiClient(object):
                 path_params,
                 query_params,
                 header_params,
+                limit=limit,
                 body=body_params,
                 post_params=form_params,
                 files=local_var_files,
@@ -221,11 +226,26 @@ class ApiClient(object):
 
         return element_data
 
+    @staticmethod
     @logging_before_and_after(logging_level=logger.debug)
-    async def request(self, method, url, query_params=None, headers=None, body=None):
+    def raise_api_exception(response: str) -> None:
+        """Raise an ApiClientError with the message changed to be more user friendly
+        :param response: the response from the API
+        """
+        replace_words = {
+            'report': 'component',
+            'app': 'menu path',
+            'business': 'workspace',
+            'dashboard': 'board',
+        }
+        for word, replacement in replace_words.items():
+            response = response.replace(word, replacement)
+        logger.error(response)
+        raise ApiClientError(response)
+
+    @logging_before_and_after(logging_level=logger.debug)
+    async def request(self, method, url, query_params=None, headers=None, body=None, limit: Optional[int] = None):
         auth = None
-        logger.debug(f'method:{method}, url: {url}, headers: {headers},'
-                     f'query params: {query_params}, body: {body}')
         if self.is_basic_auth:
             auth = ('user', self.api_key)
 
@@ -234,6 +254,7 @@ class ApiClient(object):
         elif self.is_oauth:
             headers = headers or {}
             headers.update({'Authorization': 'Bearer ' + self.access_token})
+            headers.update({'shimoku-api-version': get_distribution("shimoku_api_python").version})
 
         if method not in ['GET', 'HEAD', 'OPTIONS', 'DELETE', 'POST', 'PUT', 'PATCH']:
             raise ValueError(
@@ -247,9 +268,15 @@ class ApiClient(object):
 
             while True:  # loop until nextToken is None
 
-                async with session.request(
-                        method, url+(f'?nextToken={next_token}' if next_token else '?limit=100'),
-                        params=query_params, json=body, headers=headers) as res:
+                aux_url = url
+                if method == 'GET':
+                    aux_url += (f'?nextToken={next_token}' if next_token else f'?limit={limit if limit else 100}')
+
+                logger.debug(f'method:{method}, url: {aux_url}, headers: {headers},'
+                             f'query params: {query_params}, body: {body}')
+
+                async with session.request(method, aux_url, params=query_params, json=body, headers=headers) as res:
+                    self.call_counter += 1
                     try:
                         if 'application/json' in res.headers.get('content-type'):
                             data = await res.json()
@@ -257,10 +284,10 @@ class ApiClient(object):
                             data = await res.text()
 
                         if not res.ok:
-                            raise ApiClientError(data)
+                            self.raise_api_exception(data)
 
-                        if data.get('items'):
-                            next_token = data.get('nextToken')
+                        if 'items' in data:
+                            next_token = data.get('nextToken') if not limit else None
                             if data_res.get('items'):
                                 data_res['items'].extend(data.get('items'))
                             else:
@@ -272,7 +299,7 @@ class ApiClient(object):
                         logger.debug(data)
 
                     except Exception as e:
-                        raise ApiClientError(e)
+                        self.raise_api_exception(str(e))
 
                 if not next_token:
                     break
