@@ -3,13 +3,11 @@ from os import getenv
 from typing import Dict, List
 
 import shimoku_api_python as shimoku
-from shimoku_api_python.client import ApiClientError
 
 
 api_key: str = getenv('API_TOKEN')
 universe_id: str = getenv('UNIVERSE_ID')
 
-
 config = {
     'access_token': api_key,
 }
@@ -17,23 +15,15 @@ config = {
 s = shimoku.Client(
     config=config,
     universe_id=universe_id,
-    environment='production',
+    verbosity='DEBUG',
 )
 
 
 def test_get_universe_businesses():
     businesses: List[Dict] = (
-        s.universe.get_universe_businesses()
+        s.universes.get_universe_businesses(uuid=universe_id)
     )
     assert businesses
 
 
-def test_get_universe_app_types():
-    app_types: List[Dict] = (
-        s.universe.get_universe_app_types()
-    )
-    assert app_types
-
-
 test_get_universe_businesses()
-test_get_universe_app_types()
